/*

*/

import tree.*;

/**
   This program demonstrates how difficult it is in
   languages like Java (or C or C++) to instantiate
   a complex data structure like a tree.
   <p>
   Below is the code that instantiates the following tree.
   <pre>
                a
              / | \
             /  |  \
            b   c   d
          /  \      |
         /    \     |
        e      f    g
                  / | \
                 /  |  \
                h   i   j
                |
                |
                k
   </pre>
*/
public class Tree_Language_Example_0
{
   public static void main(String[] args)
   {
      // Here is how we can build a Tree data structure.
      Tree<String> tree = new LinkedTree<String>("a", new LinkedTree<String>("b", new LinkedTree<String>("e"), new LinkedTree<String>("f")), new LinkedTree<String>("c"), new LinkedTree<String>("d", new LinkedTree<String>("g", new LinkedTree<String>("h", new LinkedTree<String>("k")), new LinkedTree<String>("i"), new LinkedTree<String>("j"))));

      // Here is a clearer way to format the same code.
      // (Notice that this is formatted in a way analogous
      //  to the way the prettyprinter prints the tree.)
      tree = new LinkedTree<String>("a",
                                    new LinkedTree<String>("b",
                                                           new LinkedTree<String>("e"),
                                                           new LinkedTree<String>("f")),
                                    new LinkedTree<String>("c"),
                                    new LinkedTree<String>("d",
                                                           new LinkedTree<String>("g",
                                                                                  new LinkedTree<String>("h",
                                                                                                         new LinkedTree<String>("k")),
                                                                                  new LinkedTree<String>("i"),
                                                                                  new LinkedTree<String>("j"))));


      // Here is a third way to format the same code.
      tree = new LinkedTree<String>("a", new LinkedTree<String>("b", new LinkedTree<String>("e"),
                                                                     new LinkedTree<String>("f")),
                                         new LinkedTree<String>("c"),
                                         new LinkedTree<String>("d", new LinkedTree<String>("g", new LinkedTree<String>("h", new LinkedTree<String>("k")),
                                                                                             new LinkedTree<String>("i"),
                                                                                             new LinkedTree<String>("j"))));

      // Print the Tree using its toString() method.
      System.out.println( "Tree expression = " + tree + "\n" );

      // The "pretty printer" prints out the tree in a nice format.
      System.out.println( PrettyPrinter.prettyPrint( tree ) + "\n" );

      // Let us traverse the tree so that we can convince
      // ourselves that it was constructed correctly.
      // We should get "a b e f c d g h k i j".
      System.out.println( " Pre-order traversal = " + tree.preOrder() + "\n" );
      // We should get "e f b c k h i j g d a".
      System.out.println( "Post-order traversal = " + tree.postOrder() + "\n" );

      // Create dot, png, and XML files from the tree.
      Tree2dot.tree2dot(tree, "Tree_Language_Example_0");
      Tree2png.tree2png("Tree_Language_Example_0");
      Tree2xml.tree2xml(tree, "Tree_Language_Example_0");
   }
}
