/*

*/

import tree.*;
import tokenizer.*;
import treelanguage.*;

/**
   This program uses a small "Domain Specific Language" (DSL)
   to solve the problem of instantiating trees.
   <p>
   Trees are represented by strings in a simple {@code Tree}
   language and a "parser" reads these strings and builds the
   appropriate tree data structure.
   <pre>
                a
              / | \
             /  |  \
            b   c   d
          /  \      |
         /    \     |
        e      f    g
                  / | \
                 /  |  \
                h   i   j
                |
                |
                k
   </pre>
*/
public class Tree_Language_Example_1
{
   public static void main(String[] args)
   {
      // The string that represents the above tree.
      final String treeLanguage = "(a (b e f) c (d (g (h k) i j)))";

      // Build the Tree data structure from its string representation.
      final Tree<String> tree = Parse2Tree.parse( treeLanguage );

      // Print the Tree using its toString() method.
      System.out.println( "Tree expression = " + tree + "\n" );

      // The "pretty printer" prints out the tree in a nice format.
      // In a sense, it reverses the process of parsing (and building)
      // the tree, so a "pretty-printer" is sometimes called an "unparser".
      System.out.println( PrettyPrinter.prettyPrint( tree ) + "\n" );

      // Let us traverse the tree so that we can convince
      // ourselves that it was constructed correctly.
      // We should get "a b e f c d g h k i j".
      System.out.println( " Pre-order traversal = " + tree.preOrder() + "\n" );
      // We should get "e f b c k h i j g d a".
      System.out.println( "Post-order traversal = " + tree.postOrder() + "\n" );

      // Create dot, png, and XML files from the tree.
      Tree2dot.tree2dot(tree, "Tree_Language_Example_1");
      Tree2png.tree2png("Tree_Language_Example_1");
      Tree2xml.tree2xml(tree, "Tree_Language_Example_1");
   }
}
