/*

*/

import tree.*;
import tokenizer.*;
import treelanguage.*;

/**
   This program builds more trees using our tree language.
*/
public class Tree_Language_Example_2
{
   public static void main(String[] args)
   {
      final String[] treeStrings = {"(a b\n\nc\td e)", // this string tests the tokenizer
                                    "(a (b c d) (e f g))",
                                    "(a (b c) d (e (f g h i j)))",
                                    "(* (+ x y) (- z))",
                                    "(/ x (+ y z u v))",
                                    "(define (fac n) (if (== n 0) 1 (* n (fac (- n 1)))))",
                                    "(while (<= n 100) (begin (print (* n n)) (== n (+ n 2))))"
                                   };
      int i = 0;
      for (final String treeString : treeStrings)
      {
         System.out.println(i + " =============================================");

         // Build the Tree data structure from its string representation.
         final Tree<String> tree = Parse2Tree.parse( treeString );

         // Print the Tree using its toString() method.
         System.out.println( "Tree expression = " + tree + "\n" );

         // Pretty print the Tree in a nice format.
         System.out.println( PrettyPrinter.prettyPrint( tree ) + "\n" );

         // Create dot, png, and XML files from the tree.
         final String baseName = String.format("Tree_Language_Example_2.%02d", i);
         Tree2dot.tree2dot(tree, baseName);
         Tree2png.tree2png(baseName);
         Tree2xml.tree2xml(tree, baseName);
         ++i;
      }
   }
}
