/*

*/

import tree.*;
import treelanguage.*;

/**
   This program demonstrates three ways
   to instantiate the same {@link Tree}
   data structure.
   <ul>
   <li>Using the Tree language.
   <li>Using nested constructor calls.
   <li>Building the tree from the bottom up.
   </ul>
   <pre>
               apple
               /   \
              /     \
          grape     pear
                    /  \
                   /    \
                plum    peach
   </pre>
*/
public class Tree_Language_Example_3
{
   public static void main(String[] args)
   {
      // Build the Tree data structure from its string representation.
      final Tree<String> tree1 =
               Parse2Tree.parse("(apple grape (pear plum peach))");

      // Build the Tree data structure using nested constructor calls.
      final Tree<String> tree2 =
               new LinkedTree<>("apple",
                                new LinkedTree<>("grape"),
                                new LinkedTree<>("pear",
                                                 new LinkedTree<>("plum"),
                                                 new LinkedTree<>("peach")));

      // Build the Tree data structure from the "bottom up".
      final LinkedTree<String> tree7 = new LinkedTree<>("peach");
      final LinkedTree<String> tree6 = new LinkedTree<>("plum");
      final LinkedTree<String> tree5 = new LinkedTree<>("pear", tree6, tree7);
      final LinkedTree<String> tree4 = new LinkedTree<>("grape");
      final Tree<String> tree3 = new LinkedTree<>("apple", tree4, tree5);

      // Print each Tree using the Tree's toString() method.
      System.out.println("Tree expression is " + tree1);
      System.out.println("Tree expression is " + tree2);
      System.out.println("Tree expression is " + tree3);

      // The "pretty printer" prints out the tree in a nice format.
      // In a sense, it reverses the process of parsing a string
      // from the tree language so a "pretty-printer" is
      // sometimes called an "unparser". This is also sometimes
      // referred to as "serializing" the tree data structure
      // because the resulting string is a linear data structure.
      System.out.println( PrettyPrinter.prettyPrint( tree1 ) + "\n" );
      System.out.println( PrettyPrinter.prettyPrint( tree2 ) + "\n" );
      System.out.println( PrettyPrinter.prettyPrint( tree3 ) + "\n" );

      // Create dot and png files from the Tree.
      Tree2dot.tree2dot(tree1, "Tree_Language_Example_3a");
      Tree2dot.tree2dot(tree2, "Tree_Language_Example_3b");
      Tree2dot.tree2dot(tree3, "Tree_Language_Example_3c");
      Tree2png.tree2png("Tree_Language_Example_3a");
      Tree2png.tree2png("Tree_Language_Example_3b");
      Tree2png.tree2png("Tree_Language_Example_3c");
   }
}